% Asynchronous Distributed Pricing (ADP) algorithm for power control in wireless ad hoc networks
%
% =========================================================================================
% Copyright notice:
% Related Journal Reference:
% 
% J. Huang, R. Berry and M. L. Honig, "Distributed Interference Compensation for Wireless Networks", 
% IEEE Journal on Selected Areas in Communications (JSAC), vol. 24, no. 5, pp. 1074-1084, May 2006
%
% Related Conference References:
% 
% (1) J. Huang, R. Berry and M. L. Honig, "Spectrum Sharing with Distributed Interference Compensation," 
%     IEEE Dynamic Spectrum Access Networks (DySPAN), Baltimore, MD, USA, November 2005
% (2) J. Huang, R. Berry and M. L. Honig, "Performance of Distributed Utility-based Power Control for Wireless Ad Hoc Networks," 
%     IEEE MILCOM, Atlantic City, NJ, USA, October 2005
% (3) J. Huang, R. Berry and M. L. Honig, "Distributed Interference Compensation for Multi-Channel Wireless Networks," 
%     Annual Allerton Conference on Communication, Control and Computing, Monticello, IL, USA, September, 2005
% (4) J. Huang, R. Berry and M. L. Honig, "A Game Theoretic Analysis of Distributed Power Control for Spread Spectrum Ad Hoc Networks," 
%     IEEE International Symposium on Information Theory (ISIT), Adelaide, Australia, September 2005
%
% All rights belong to Jianwei Huang (email: ADP.20.jianweiece@antichef.com), Randall Berry, Michael Honig.
%
% This simulation code can be freely modified and distributed, as long as the the copyright notice
% section is kept unchanged. Any comments/suggestions are welcome. However, the original authors of 
% the code are not responsible for any damages caused by this code.
%
% If this simulation code (or its variation) is used in adademic research, one or more of the 
% above journal/conference references should be appropriately cited. 
%
% Author: Jianwei Huang
% Current version: March 31, 2007
% ========================================================================================

clear all;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% M: number of node pairs (users)
% B: bandwidth
% pMax: maximum transmission power of each transmitter (same for every pair)
% hArray: path attenuation matrix
% thetaArray: users' utility parameter vector
% gridLength: the whole grid has (gridlength)^2 positions
% gridUnit: the length of a smallest grid
% transReceDis: the receiver is within a grid of (2*tarnsReceDis+1)^2 with
%               the transmitter as the center.
% attenFactor: the attenuation between two nodes is distance^(-attenFactor)
% thetaArray: users' weights
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Mite = 10:5:30; % number of users
TotalRealization = 20; % number of random topologies for each parameter setup

for iM = 1:length(Mite);
    
M = Mite(iM);

% M = 10; % number of users
% kappa = 0.001; % stepsize for the gradient method on power

gridUnit = 0.2; % the lengt of a unit grid (users are put on the grid)
attenFactor = 4;

% the length of the network is set so that the attenuation from the southeast 
% corner to the southwest corner is 40dB, and the transmitter-receiver max
% attenuation is 20dB.
gridLength = ceil( 10^(4/attenFactor) / gridUnit);
transReceDis = ceil( 10^(2/attenFactor) / gridUnit); 

for iRealization = 1: TotalRealization;
% randomly generate network topology with transimitters uniformly distributed
[ distanceArray,pairPositionArray ] = funGridSetupTransmitterUniform( ...
    gridLength,transReceDis,M);

B = 128;
pMax = 1;
n0 = 1e-4;
thetaArray = ones(1,M); 

iniType = 3; % Initialization type of power and price
stopEps = 1e-4; % the stopping criterion for the relatively difference in
                % aggregate utility
GradientMaxIte = 3000; % maximum number of iterations allowed for gradient-based method

[ powerMGArray,sinrMGArray,utilityMGArray,priceConvergeProcessArray,powerConvergeProcessArray,TotalIteNum]= ...
    funADPAlgorithmLogU( M, B, pMax, n0, thetaArray, distanceArray, gridUnit, ...
                    attenFactor,iniType,stopEps);

utilityTotal(iM,iRealization) = sum(utilityMGArray);

end; % end of iteration of iRealization

end; % end of iteration of iM

utilityAvg = sum(utilityTotal,2)/TotalRealization;

% plot the convergence speed ratio between ADP algorithm and gradient-based algorithm
figure(1); clf reset; hold on;

plot(Mite,squeeze(utilityAvg),'r-o','MarkerSize',10);

set(gca,'FontSize',16);
ylabel('\fontsize{16}Users'' Total Utility');
xlabel('\fontsize{16}Number of Users: M');



