function [ distanceArray,pairPositionArray ] = funGridSetupTransmitterUniform( gridLength,transReceDis,M )
% Set up a grid, and uniformly and randomly place the transmitters (not in previously
% divided squres), and place the receivers within some area of the transmitters.

% =========================================================================================
% Copyright notice:
% Related Journal Reference:
% 
% J. Huang, R. Berry and M. L. Honig, "Distributed Interference Compensation for Wireless Networks", 
% IEEE Journal on Selected Areas in Communications (JSAC), vol. 24, no. 5, pp. 1074-1084, May 2006
%
% Related Conference References:
% 
% (1) J. Huang, R. Berry and M. L. Honig, "Spectrum Sharing with Distributed Interference Compensation," 
%     IEEE Dynamic Spectrum Access Networks (DySPAN), Baltimore, MD, USA, November 2005
% (2) J. Huang, R. Berry and M. L. Honig, "Performance of Distributed Utility-based Power Control for Wireless Ad Hoc Networks," 
%     IEEE MILCOM, Atlantic City, NJ, USA, October 2005
% (3) J. Huang, R. Berry and M. L. Honig, "Distributed Interference Compensation for Multi-Channel Wireless Networks," 
%     Annual Allerton Conference on Communication, Control and Computing, Monticello, IL, USA, September, 2005
% (4) J. Huang, R. Berry and M. L. Honig, "A Game Theoretic Analysis of Distributed Power Control for Spread Spectrum Ad Hoc Networks," 
%     IEEE International Symposium on Information Theory (ISIT), Adelaide, Australia, September 2005
%
% All rights belong to Jianwei Huang (email: ADP.20.jianweiece@antichef.com), Randall Berry, Michael Honig.
%
% This simulation code can be freely modified and distributed, as long as the the copyright notice
% section is kept unchanged. Any comments/suggestions are welcome. However, the original authors of 
% the code are not responsible for any damages caused by this code.
%
% If this simulation code (or its variation) is used in adademic research, one or more of the 
% above journal/conference references should be appropriately cited. 
%
% Author: Jianwei Huang
% Current version: March 31, 2007
% ========================================================================================
  
% clear;
% gridLength = 16;
% gridUnit = 0.1;
% transReceDis = 2;
% M = 16;
% attenFactor = 4;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% gridLength: the whole grid has (gridlength)^2 positions
% gridUnit: the length of a smallest grid
% transReceDis: the receiver is within a square of (2*tarnsReceDis+1)^2 points with
%               the transmitter as the center.
% M: the number of transmitter-receiver pairs (users)
% attenFactor: the attenuation between two nodes is distance^(-attenFactor)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% gridPositionArray: position indicator matrix of the whole grid                
%                    0: no node 
%                    1: a transmitter
%                    2: a receiver
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
gridPositionArray = zeros(gridLength,gridLength);  % initialization
	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% pairPositionArray: position matrix of the pairs (Mx4), with M rows and 4 columns:
%                    (x-axis of transmitter, y-axis of transmitter, x-axis
%                    of receiver, y-axis of receiver)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pairPositionArray = zeros(M,4);  % initialization


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% we place the transmitters randomly in the network by uniformly and randomly choose
% their indivisual locations.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% random permutation of all the positions
randPermGridPosition = randperm(gridLength^2);
% pick M positions for the transmitters
gridPositionArray(randPermGridPosition(1:M)) = 1;
% find the x and y positions for the transmitters
[pairPositionArray(:,1),pairPositionArray(:,2)] ...
    = find(gridPositionArray == 1);

% Determine the area of the receivers:
% receBasePointArray: M X 2 matrix of the location of the southwest corner of the receivers
% receUpPointArray: M X 2 matrix of the location of the northeast corner of the receivers
receBasePointArray(:,1) = max(1,pairPositionArray(:,1)-transReceDis);
receBasePointArray(:,2) = max(1,pairPositionArray(:,2)-transReceDis);
receUpPointArray(:,1)   = min(gridLength,pairPositionArray(:,1)+transReceDis);
receUpPointArray(:,2)   = min(gridLength,pairPositionArray(:,2)+transReceDis);    

for ipair = 1:M;
  
% the grid that the receiver of user i belongs to
  receGrid = gridPositionArray(receBasePointArray(ipair,1):receUpPointArray(ipair,1),...
                             receBasePointArray(ipair,2): ...
                             receUpPointArray(ipair,2));

% available position (x-axis and y-axis) for the receiver of user i
  [availX,availY] = find(receGrid == 0);
      
  if length(availX) == 0
    fprintf('\nNo available position for the receiver');
    exit;
  end;      
  
% randomly pick the location for the receiver in the available positions
  randomPosition = ceil(length(availX)*rand(1,1));  
  receX  = availX(randomPosition);
  receY  = availY(randomPosition); 
  
% the exact location of the receiver of user i
  pairPositionArray(ipair,3:4)= [receBasePointArray(ipair,1)+receX-1, ...
                      receBasePointArray(ipair,2)+receY-1];
  gridPositionArray(pairPositionArray(ipair,3),pairPositionArray(ipair,4)) = 2;
        
end; % end of loop of ipair    
    
fprintf(['\nTopology generated: M = ',num2str(M)]);
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% distanceArray: the distance between the transmitters and the receivers (units of grids)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

distanceArray = zeros(M,M); % initialization
    
for ipair = 1:M
% the position of user i's transmitter
  transmitterPositionTemp = pairPositionArray(ipair,1:2); 
  
  distanceArray(ipair,:) = ...
      reshape(sqrt(sum((transmitterPositionTemp(ones(M,1),:)-pairPositionArray(:,3:4)).^2,2)), 1,M);
end;         

    
        
        
	
